% Script to demonstration of the shiftNMF algorithm
%
% Copyright (C) Morten Mrup, Kristoffer Hougaard Madsen and Technical University of Denmark, 
% April 2007
% 
% Notice, due to local minima the algorithm does not always identify the
% correct solution.

clear all;
maxdelay=100;
usecomp=[1 2 3];
sigmaN=0;

xx=linspace(0,10,1000);
H(1,:)=[normpdf(xx,2,0.3)+normpdf(xx,5,0.2)+normpdf(xx,7,.5)*.5]';
H(2,:)=[pdf('rayl',xx,0.2)]'+normpdf(xx,4,0.1)';
H(3,:)=[fliplr(pdf('rayl',xx,0.4))+normpdf(xx,3,0.4)]';
% zero pad around delays
H=[zeros(size(H,1),2*maxdelay), H, zeros(size(H,1),2*maxdelay)];

W=rand(9,length(usecomp));
T=maxdelay*2*(rand(size(W))-0.5); 

% Align H and T
N=size(H,2);
f=j*2*pi*[0:N-1]/N;
tmean=mean(T);
T=T-repmat(tmean,[size(T,1),1]);
Hf=fft(H,[],2);
Hf=Hf(:,1:floor(size(Hf,2)/2)+1);
f=f(1:size(Hf,2));
Hf=Hf.*exp(tmean'*f);
Hft=[Hf conj(Hf(:,end-1:-1:2))];
H=real(ifft(Hft,[],2));
H(H<0)=0;

% Normalize W with respect to H
d=sqrt(sum(W.^2,1));
H=H.*repmat(d',[1,size(H,2)]);
W=W./repmat(d,[size(W,1),1]);


% Generate simulated data from factors
Hf=fft(H,[],2);
Hf=Hf(:,1:floor(size(Hf,2)/2)+1);
N=size(H,2);
f=j*2*pi*[0:N-1]/N;
f=f(1:size(Hf,2));
for i=1:size(W,1)
   Hft=Hf(usecomp,:).*exp(T(i,usecomp)'*f); 
   Hft=[Hft conj(Hft(:,end-1:-1:2))];
   Ht=real(ifft(Hft,[],2));
   X(i,:)=W(i,usecomp)*Ht;
end
X(X<0)=0;

Htrue=H(usecomp,:);
Wtrue=W(:,usecomp);
Ttrue=T(:,usecomp);
opts.maxiter=500;
opts.runit=1;
%opts.lambda=1;
%opts.smoothtype='regularization';
%opts.smoothnorm='L1';

% Add noise 
NS=norm(X,'fro')^2;
N=sigmaN*rand(size(X));
SNR=NS/norm(N,'fro')^2;
X=X+N;

% Estimate model
noc=length(usecomp);
[W, H, T, varexpl, cost]=ShiftNMF(X,noc,opts);

% Plot the results
figure;
for k=1:noc
    subplot(noc,3,(k-1)*3+1)
    hinton(Wtrue(:,k));
    title(['True W_' num2str(k)]);
    axis tight;
   
    subplot(noc,3,(k-1)*3+2)
    plot(Htrue(k,:));
    title(['True H_' num2str(k)]);
    axis tight;

    subplot(noc,3,(k-1)*3+3)
    barh(Ttrue(:,k));
    title(['True T_' num2str(k)]);
    axis tight;      
end

ind=[1 2 3];
figure;
for k=1:noc
    subplot(noc,3,(k-1)*3+1)
    hinton(W(:,ind(k)));
    title(['Est. W_' num2str(k)]);
    axis tight;
   
    subplot(noc,3,(k-1)*3+2)
    plot(H(ind(k),:));
    title(['Est. H_' num2str(k)]);
    axis tight;

    subplot(noc,3,(k-1)*3+3)
    barh(T(:,ind(k)));
    title(['Est. T_' num2str(k)]);
    axis tight;  
end

